﻿Imports System.Drawing

Namespace RealTimeChessFrameWork
    Friend Class clsCommon


        Private Sub New()
        End Sub



        ''' <summary>
        ''' ImageLoad
        ''' 駒の色と種類に応じた画像をリソースより取得する
        ''' </summary>
        ''' <param name="design">駒の種類</param>
        ''' <param name="color">駒の色</param>
        Public Shared Function GetPieceImage(ByVal design As Design, ByVal color As PieceColor) As Bitmap

            Dim bmp As Bitmap = Nothing

            If color = PieceColor.Black Then
                Select Case design
                    Case design.Pawn
                        bmp = New Bitmap(My.Resources.b_pawn)
                        Exit Select
                    Case design.Rook
                        bmp = New Bitmap(My.Resources.b_rook)
                        Exit Select
                    Case design.Knight
                        bmp = New Bitmap(My.Resources.b_knight)
                        Exit Select
                    Case design.Bishop
                        bmp = New Bitmap(My.Resources.b_bishop)
                        Exit Select
                    Case design.Queen
                        bmp = New Bitmap(My.Resources.b_queen)
                        Exit Select
                    Case design.King
                        bmp = New Bitmap(My.Resources.b_king)
                        Exit Select
                End Select
            Else
                Select Case design
                    Case design.Pawn
                        bmp = New Bitmap(My.Resources.w_pawn)
                        Exit Select
                    Case design.Rook
                        bmp = New Bitmap(My.Resources.w_rook)
                        Exit Select
                    Case design.Knight
                        bmp = New Bitmap(My.Resources.w_knight)
                        Exit Select
                    Case design.Bishop
                        bmp = New Bitmap(My.Resources.w_bishop)
                        Exit Select
                    Case design.Queen
                        bmp = New Bitmap(My.Resources.w_queen)
                        Exit Select
                    Case design.King
                        bmp = New Bitmap(My.Resources.w_king)
                        Exit Select
                End Select
            End If

            Return bmp

        End Function



        ''' <summary>
        ''' 画像から背景色を切り抜いたRegionを生成する
        ''' </summary>
        ''' <param name="bitmap">画像</param>
        Public Shared Function GetRegionFromBitmap(ByVal bitmap As Bitmap) As Region
            If Not (bitmap.PixelFormat = System.Drawing.Imaging.PixelFormat.Format24bppRgb _
                    OrElse bitmap.PixelFormat = System.Drawing.Imaging.PixelFormat.Format32bppArgb _
                    OrElse bitmap.PixelFormat = System.Drawing.Imaging.PixelFormat.Format32bppRgb) Then
                Return Nothing
            End If

            ' 24bitか32bit以外は無理
            Dim channels As Integer = Image.GetPixelFormatSize(bitmap.PixelFormat) \ 8
            Dim offset As Integer = 0
            Dim count As Integer = channels

            'ビットマップから生データを取得
            Dim rectBitmap As New Rectangle(0, 0, bitmap.Width, bitmap.Height)
            Dim data As System.Drawing.Imaging.BitmapData = bitmap.LockBits(rectBitmap, _
                                                                            Imaging.ImageLockMode.ReadOnly, _
                                                                            bitmap.PixelFormat)
            Dim values As Byte() = New Byte(data.Height * data.Stride - 1) {}
            System.Runtime.InteropServices.Marshal.Copy(data.Scan0, values, 0, data.Height * data.Stride)
            bitmap.UnlockBits(data)

            '透明にする色を取得
            Dim transparent As Byte() = New Byte(count - 1) {}
            If count = 1 Then
                transparent(0) = 0
            Else
                For i As Integer = 0 To count - 1
                    transparent(i) = values(i)
                Next
            End If

            Dim region As New Region()
            region.MakeEmpty()

            Dim rect As New Rectangle()
            For y As Integer = 0 To data.Height - 1
                Dim offsetY As Integer = y * data.Stride
                For x As Integer = 0 To data.Width - 1
                    For c As Integer = 0 To count - 1
                        If transparent(c) <> values(offsetY + x * channels + c + offset) Then
                            '透過色ではない
                            rect.X = x
                            '透過色ではないスタート地点を保存
                            x += 1
                            While x < data.Width
                                For cc = 0 To count - 1
                                    If transparent(cc) <> values(offsetY + x * channels + cc + offset) Then

                                        GoTo [CONTINUE]
                                        '透過色ではないので続ける
                                    End If
                                Next
                                Exit While
[CONTINUE]:
                                '次の透過色を見つけたので脱出
                                x += 1
                            End While
                            rect.Width = x - rect.X
                            rect.Y = y
                            rect.Height = 1

                            '透過色でない部分をリージョンに追加する
                            region.Union(rect)
                            Exit For
                        End If
                    Next
                Next
            Next

            Return region
        End Function
    End Class



End Namespace