﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Drawing
Imports System.Windows.Forms


Namespace RealTimeChessFrameWork

    '' <summary>
    '' 各Piece
    '' Drag&Dropにて移動可能
    '' </summary>
    Friend Class clsPiece
        Inherits PictureBox
        Private _Idx As Integer
        Private _Owner As clsPlayer        '持ち主
        Private _Design As Design        '駒の絵柄
        Private _PieceColor As PieceColor        '駒の色
        Private _PieceDirection As PieceDirection        '駒の進行方向
        Private _Place As Point        '駒の場所
        Private _IsDragging As Boolean        'ドラッグ中かどうか
        Private _BeforeMovePoint As Point        '移動前の座標
        Private _IsMovable As Boolean        '動かせるかどうか
        Private _Grid As clsGrid        '盤面
        Private Tim As Timer        'タイマー
        Private MovableDirection As New List(Of Point)()        '突き進める方向
        Private MovablePoint As New List(Of Point)()        '固定で進める場所
        Private NowMovablePoint As New List(Of Point)()        '今動ける場所
        Private NowGettablePoint As New List(Of Point)()        '今取れる場所


        ''' <summary>
        ''' 駒の番号 通信プレイで相手にどの駒か伝えるために使用
        ''' </summary>
        Public Property Idx() As Integer
            Get
                Return _Idx
            End Get
            Set(ByVal value As Integer)
                _Idx = value
            End Set
        End Property


        ''' <summary>
        ''' 駒の持ち主 自分か相手
        ''' </summary>
        Public Property Owner() As clsPlayer
            Get
                Return _Owner
            End Get
            Private Set(ByVal value As clsPlayer)
                _Owner = value
            End Set
        End Property


        ''' <summary>
        ''' 駒の模様
        ''' King,Queen...
        ''' </summary>
        Public Property Design() As Design
            Get
                Return _Design
            End Get
            Private Set(ByVal value As Design)
                _Design = value
            End Set
        End Property


        ''' <summary>
        ''' 駒の色
        ''' White or Black
        ''' </summary>
        Public Property PieceColor() As PieceColor
            Get
                Return _PieceColor
            End Get
            Private Set(ByVal value As PieceColor)
                _PieceColor = value
            End Set
        End Property


        ''' <summary>
        ''' 駒の進行方向
        ''' Up or Down
        ''' </summary>
        Public Property PieceDirection() As PieceDirection
            Get
                Return _PieceDirection
            End Get
            Private Set(ByVal value As PieceDirection)
                _PieceDirection = value
            End Set
        End Property


        ''' <summary>
        ''' 駒の場所
        ''' </summary>
        Public Property Place() As Point
            Get
                Return _Place
            End Get
            Set(ByVal value As Point)
                _Place = value
                Me.Left = value.X * clsConst.CellSize.Width
                Me.Top = value.Y * clsConst.CellSize.Height
            End Set
        End Property


        ''' <summary>
        ''' マウスドラッグ中かどうか
        ''' </summary>
        Public Property IsDragging() As Boolean
            Get
                Return _IsDragging
            End Get
            Set(ByVal value As Boolean)
                _IsDragging = value
            End Set
        End Property


        ''' <summary>
        ''' マウスドラッグ前の座標
        ''' </summary>
        Public Property BeforeMovePoint() As Point
            Get
                Return _BeforeMovePoint
            End Get
            Set(ByVal value As Point)
                _BeforeMovePoint = value
            End Set
        End Property


        ''' <summary>
        ''' 動かせるかどうか
        ''' </summary>
        Public Property IsMovable() As Boolean
            Get
                Return _IsMovable
            End Get
            Set(ByVal value As Boolean)
                If Not value Then
                    Me.Tim.Start()
                End If
                'falseになったらTimerStart
                _IsMovable = value
            End Set
        End Property


        ''' <summary>
        ''' 盤面
        ''' </summary>
        Public Property Grid() As clsGrid
            Get
                Return _Grid
            End Get
            Private Set(ByVal value As clsGrid)
                _Grid = value
            End Set
        End Property


        ''' <summary>
        ''' 配置できる場所かどうか
        ''' </summary>
        ''' <param name="p">配置先</param>
        Public Function IsMovablePoint(ByVal p As Point) As Boolean
            For Each pp As Point In NowMovablePoint
                If pp.X = p.X AndAlso pp.Y = p.Y Then
                    Return True
                End If
            Next
            Return False
        End Function


        ''' <summary>
        ''' 敵の駒を取得できる場所かどうか
        ''' </summary>
        ''' <param name="p">配置先</param>
        Public Function IsGettable(ByVal p As Point) As Boolean
            For Each pp As Point In NowGettablePoint
                If pp.X = p.X AndAlso pp.Y = p.Y Then
                    Return True
                End If
            Next
            Return False
        End Function



        ''' <summary>
        ''' コンストラクタ
        ''' </summary>
        ''' <param name="design">駒の模様</param>
        ''' <param name="color">駒の色</param>
        ''' <param name="pt">駒の場所</param>
        ''' <param name="dir">駒の進行方向</param>
        ''' <param name="grid">盤面</param>
        ''' <param name="owner">持ち主</param>
        Public Sub New(ByVal design As Design, ByVal color As PieceColor, ByVal pt As Point, ByVal dir As PieceDirection, ByVal grid As clsGrid, ByVal owner As clsPlayer)

            Me.Design = design            '模様
            Me.PieceColor = color            '白黒
            Me.Place = pt            '場所
            Me.PieceDirection = dir            '進行方向
            Me.Grid = grid            '盤面
            Me.Owner = owner            '操作可能か否か
            Me.IsDragging = False
            Me.BeforeMovePoint = Me.Place
            Me.IsMovable = True
            Me.Size = clsConst.CellSize
            Me.MovingAreaInit()

            Dim bmp As Bitmap = clsCommon.GetPieceImage(design, color)
            Me.Image = bmp
            Me.Region = clsCommon.GetRegionFromBitmap(bmp)
            Me.Location = New Point(Place.X * clsConst.CellSize.Width, Place.Y * clsConst.CellSize.Height)

            Tim = New Timer()
            Tim.Interval = 3000
            AddHandler Tim.Tick, New EventHandler(AddressOf Tim_Tick)
            AddHandler Me.MouseDown, New MouseEventHandler(AddressOf Piece_MouseDown)
            AddHandler Me.MouseUp, New MouseEventHandler(AddressOf Piece_MouseUp)
            AddHandler Me.MouseMove, New MouseEventHandler(AddressOf Piece_MouseMove)

        End Sub



        Private Sub Piece_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
            'ドラッグしていなければ何もしない
            If Not Me.IsDragging Then
                Return
            End If
            Me.Left += e.X - BeforeMovePoint.X
            Me.Top += e.Y - BeforeMovePoint.Y
        End Sub



        Private Sub Piece_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs)
            IsDragging = False
            Dim x As Integer = (Me.Left + clsConst.CellSize.Width \ 2) \ clsConst.CellSize.Width
            Dim y As Integer = (Me.Top + clsConst.CellSize.Height \ 2) \ clsConst.CellSize.Height
            Dim NowPoint As New Point(x, y)
            If x > 7 OrElse _
                y > 7 OrElse _
                x < 0 OrElse _
                y < 0 OrElse _
                (Not IsMovablePoint(NowPoint) AndAlso Not IsGettable(NowPoint)) Then

                'はみ出ている場合　or　置けない場合は元に戻す
                Me.Left = Me.Place.X * clsConst.CellSize.Width
                Me.Top = Me.Place.Y * clsConst.CellSize.Height

            Else

                'はみ出ていなければそこに移動する
                Me.Left = NowPoint.X * clsConst.CellSize.Width
                Me.Top = NowPoint.Y * clsConst.CellSize.Height

                Me.Grid.MovePiece(Me, NowPoint)
            End If

            'Cellの色分けを消す
            Me.Grid.AllCellClear()
        End Sub



        Private Sub Piece_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs)
            'System.Diagnostics.Debug.Print(Me.Place.ToString())


            '動かせなければ何もしない
            If Not Me.IsMovable Then
                Return
            End If
            '自分の駒でなければ何もしない
            If Not Me.Owner.IsMovable Then
                Return
            End If
            IsDragging = True
            BeforeMovePoint = New Point(e.X, e.Y)
            Me.CalculateMovableArea()
            Me.ShowMovableArea()
            Me.Grid.ParentPictureBox.Refresh()

        End Sub



        Private Sub Tim_Tick(ByVal sender As Object, ByVal e As EventArgs)
            Me.IsMovable = True
            Tim.[Stop]()
            Me.Refresh()
        End Sub



        Public Sub CalculateMovableArea()
            'pの移動可能なCellの座標をNowMovablePointかNowGettablePointに格納する
            NowMovablePoint.Clear()
            NowGettablePoint.Clear()

            For Each point As Point In MovableDirection
                '探す方向
                Dim search As New Point(Place.X, Place.Y)
                Dim flag As Boolean = True
                'まだ先を探せるかどうか
                Do
                    search.X += point.X
                    search.Y += point.Y

                    If search.X > 7 OrElse search.Y > 7 OrElse search.X < 0 OrElse search.Y < 0 Then
                        'はみ出ている場合は何もせずに終了
                        flag = False
                    Else
                        If Grid(search.X, search.Y).Piece Is Nothing Then
                            '駒がなければ
                            NowMovablePoint.Add(New Point(search.X, search.Y))
                        ElseIf Grid(search.X, search.Y).Piece.Owner.Name <> Owner.Name Then
                            '敵の駒なら
                            NowGettablePoint.Add(New Point(search.X, search.Y))
                            'Grid[search.X, search.Y].CellColor = CellColor.Blue;
                            flag = False
                        Else
                            '見方の駒なら
                            flag = False
                        End If
                    End If
                Loop While flag
            Next
            For Each point As Point In MovablePoint

                '探す方向
                Dim search As New Point(Place.X, Place.Y)
                '自分の場所
                If Me.PieceDirection = PieceDirection.Down Then
                    search.X += point.X
                    search.Y += point.Y
                Else
                    search.X -= point.X
                    search.Y -= point.Y
                End If
                'はみ出ている場合は何もせずに終了
                If search.X > 7 OrElse search.Y > 7 OrElse search.X < 0 OrElse search.Y < 0 Then
                Else
                    If Grid(search.X, search.Y).Piece Is Nothing Then
                        '駒がなければ
                        'Grid[search.X, search.Y].CellColor = CellColor.Red;
                        NowMovablePoint.Add(New Point(search.X, search.Y))
                    ElseIf Grid(search.X, search.Y).Piece.Owner.Name <> Owner.Name Then
                        '敵の駒なら
                        'Grid[search.X, search.Y].CellColor = CellColor.Blue;
                        NowGettablePoint.Add(New Point(search.X, search.Y))
                        '見方の駒なら
                    Else
                    End If


                End If
            Next


            If Me.Design = Design.Pawn Then
                Dim search As New Point(Place.X, Place.Y)
                '自分の場所
                '1歩前
                Dim point As New Point(0, 1)
                If PieceDirection = PieceDirection.Down Then
                    search.X += point.X
                    search.Y += point.Y
                Else
                    search.X -= point.X
                    search.Y -= point.Y
                End If
                'はみ出ている場合は何もせずに終了
                If search.X > 7 OrElse search.Y > 7 OrElse search.X < 0 OrElse search.Y < 0 Then
                Else
                    If Grid(search.X, search.Y).Piece Is Nothing Then
                        '駒がなければ移動可能
                        NowMovablePoint.Add(New Point(search.X, search.Y))
                        '敵の駒なら進めない
                    ElseIf Grid(search.X, search.Y).Piece.Owner.Name <> Owner.Name Then
                        '見方の駒なら進めない
                    Else
                    End If
                End If

                If Me.Place.Y = 1 OrElse Me.Place.Y = 6 Then
                    '移動していないときのみ2歩進める
                    search = New Point(Place.X, Place.Y)
                    point = New Point(0, 2)
                    If PieceDirection = PieceDirection.Down Then
                        search.X += point.X
                        search.Y += point.Y
                    Else
                        search.X -= point.X
                        search.Y -= point.Y
                    End If
                    'はみ出ている場合は何もせずに終了
                    If search.X > 7 OrElse search.Y > 7 OrElse search.X < 0 OrElse search.Y < 0 Then
                    Else
                        If Grid(search.X, search.Y).Piece Is Nothing Then
                            '駒がなければ移動可能
                            NowMovablePoint.Add(New Point(search.X, search.Y))
                            '敵の駒なら進めない
                        ElseIf Grid(search.X, search.Y).Piece.Owner.Name <> Owner.Name Then
                            '見方の駒なら進めない
                        Else
                        End If
                    End If
                End If
                '斜め前に敵がいれば取れる
                '右前
                search = New Point(Place.X, Place.Y)
                point = New Point(1, 1)
                If PieceDirection = PieceDirection.Down Then
                    search.X += point.X
                    search.Y += point.Y
                Else
                    search.X -= point.X
                    search.Y -= point.Y
                End If
                If search.X > 7 OrElse search.Y > 7 OrElse search.X < 0 OrElse search.Y < 0 Then
                Else
                    'はみ出ている場合は何もせずに終了
                    If Grid(search.X, search.Y).Piece Is Nothing Then
                        '駒がなくても移動不可能
                    ElseIf Grid(search.X, search.Y).Piece.Owner.Name <> Owner.Name Then
                        NowGettablePoint.Add(New Point(search.X, search.Y))
                    Else
                        '敵の駒なら移動可能
                        '見方の駒なら進めない
                    End If
                End If
                '左前
                search = New Point(Place.X, Place.Y)
                point = New Point(-1, 1)
                If PieceDirection = PieceDirection.Down Then
                    search.X += point.X
                    search.Y += point.Y
                Else
                    search.X -= point.X
                    search.Y -= point.Y
                End If
                If search.X > 7 OrElse search.Y > 7 OrElse search.X < 0 OrElse search.Y < 0 Then
                Else
                    'はみ出ている場合は何もせずに終了
                    If Grid(search.X, search.Y).Piece Is Nothing Then
                        '駒がなくても移動不可能
                    ElseIf Grid(search.X, search.Y).Piece.Owner.Name <> Owner.Name Then
                        NowGettablePoint.Add(New Point(search.X, search.Y))
                    Else
                        '敵の駒なら移動可能
                        '見方の駒なら進めない         
                    End If
                End If
            End If
        End Sub



        Public Sub ShowMovableArea()
            For Each p As Point In NowMovablePoint
                Me.Grid(p.X, p.Y).CellColor = CellColor.Red
            Next
            For Each p As Point In NowGettablePoint
                Me.Grid(p.X, p.Y).CellColor = CellColor.Blue
            Next
        End Sub



        Public Sub Draw(ByVal g As Graphics)
            Me.Refresh()
            If Not IsMovable Then
                Dim br As New SolidBrush(Color.FromArgb(200, Color.White))
                Me.CreateGraphics().FillRectangle(br, 0, 0, clsConst.CellSize.Width, clsConst.CellSize.Height)
            End If
        End Sub



        Private Sub MovingAreaInit()
            '進める場所を定義しておく
            Select Case Me.Design
                Case Design.Pawn
                    'MovablePoint.Add(new Point(0, 1));
                    'MovablePoint.Add(new Point(0, 2));
                    'MovablePoint.Add(new Point(1, 1));
                    'MovablePoint.Add(new Point(-1, 1));
                    Exit Select
                Case Design.Rook
                    MovableDirection.Add(New Point(0, 1))
                    MovableDirection.Add(New Point(0, -1))
                    MovableDirection.Add(New Point(1, 0))
                    MovableDirection.Add(New Point(-1, 0))
                    Exit Select
                Case Design.Knight
                    MovablePoint.Add(New Point(2, 1))
                    MovablePoint.Add(New Point(-2, 1))
                    MovablePoint.Add(New Point(-1, 2))
                    MovablePoint.Add(New Point(-1, -2))
                    MovablePoint.Add(New Point(-2, -1))
                    MovablePoint.Add(New Point(2, -1))
                    MovablePoint.Add(New Point(1, -2))
                    MovablePoint.Add(New Point(1, 2))
                    Exit Select
                Case Design.Bishop
                    MovableDirection.Add(New Point(1, 1))
                    MovableDirection.Add(New Point(1, -1))
                    MovableDirection.Add(New Point(-1, -1))
                    MovableDirection.Add(New Point(-1, 1))
                    Exit Select
                Case Design.Queen
                    MovableDirection.Add(New Point(0, 1))
                    MovableDirection.Add(New Point(0, -1))
                    MovableDirection.Add(New Point(1, 0))
                    MovableDirection.Add(New Point(-1, 0))
                    MovableDirection.Add(New Point(1, 1))
                    MovableDirection.Add(New Point(1, -1))
                    MovableDirection.Add(New Point(-1, -1))
                    MovableDirection.Add(New Point(-1, 1))
                    Exit Select
                Case Design.King
                    MovablePoint.Add(New Point(0, 1))
                    MovablePoint.Add(New Point(0, -1))
                    MovablePoint.Add(New Point(1, 0))
                    MovablePoint.Add(New Point(-1, 0))
                    MovablePoint.Add(New Point(1, 1))
                    MovablePoint.Add(New Point(1, -1))
                    MovablePoint.Add(New Point(-1, -1))
                    MovablePoint.Add(New Point(-1, 1))
                    Exit Select
            End Select

        End Sub
    End Class
End Namespace