﻿Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms
Imports RealTimeChess.RealTimeChessFrameWork
Imports Uchi.Hobby.Connection

Partial Friend Class frmMain
    Inherits Form



    ''' <summary>
    ''' 状況を相手に伝えるためのKeyword
    ''' </summary>
    Public Enum enMsgKey
        READY = 0
        '準備完了
        MOVE
        '移動
        [GET]
        '駒取得
        WIN
        '勝利
    End Enum



    ''' <summary>
    ''' 現在のゲームの状況
    ''' </summary>
    Public Enum enState
        Disconnected = 0
        '未接続
        WaitClient
        'クライアント接続待ち
        WaitOwnStart
        '自分のスタート押し待ち
        WaitRivalStart
        'ライバルのスタート待ち
        StartReady
        'スタートを押して開始待ち
        PlayingGame
        'ゲーム中
        GameSet
        'ゲーム終了
    End Enum



    Public Sub New()
        InitializeComponent()
    End Sub



    Private _State As enState
    Private _Grid As clsGrid
    Private _Own As clsPlayer
    Private _Rival As clsPlayer
    Private _tcp As TcpConnection
    Private _Tim As New Timer()
    Private _CountDown As Integer
    Private _Tim2 As New Timer()
    Private _FontDepth As Integer



    Public WriteOnly Property OwnWin() As Boolean
        Set(ByVal value As Boolean)
            _Own.IsWinner = value
            If value Then
                'this.State = enState.GameSet;
                Me.SetState(enState.GameSet)
            End If
        End Set
    End Property


    ''' <summary>
    ''' 状態遷移
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Property State() As enState
        Get
            Return _State
        End Get
        Set(ByVal value As enState)
            If InvokeRequired Then
                Throw New Exception("SetState使って～")
            End If

            Select Case value
                Case enState.Disconnected
                    '未接続
                    _Grid.AllCellPiecesClear()
                    'Cell内のPieceの参照を削除
                    _Own = Nothing
                    'Playerを削除
                    _Rival = Nothing
                    '対戦相手を削除
                    If _tcp IsNot Nothing Then
                        '通信していれば切断
                        _tcp.Dispose()
                        _tcp = Nothing
                    End If
                    Me.Init_pnlSetting()
                    '設定画面を表示
                    Me.lblMessage.Text = "Mode , Ip , Port を選択し、" & vbCr & vbLf & "STARTを押してください。"
                    Me.Refresh()
                    Exit Select
                Case enState.WaitClient
                    'クライアント待ち
                    Me.lblMessage.Text = "Clientの接続を待っています。"
                    Me.radServer.Enabled = False
                    Me.radClient.Enabled = False
                    Me.txtPort.Enabled = False
                    Me.btnStartCancel.Enabled = True
                    Me.btnStartCancel.Text = "Cancel"
                    Exit Select
                Case enState.WaitOwnStart
                    '自分のスタート待ち
                    Me.pnlSetting.Visible = True
                    Me.radServer.Visible = False
                    Me.radClient.Visible = False
                    Me.lblCaption1.Visible = False
                    Me.lblCaption2.Visible = False
                    Me.txtIP.Visible = False
                    Me.txtPort.Visible = False
                    Me.btnStartCancel.Text = "START"
                    Me.btnStartCancel.Enabled = True
                    Me.lblMessage.Text = "準備ができたら" & vbCr & vbLf & "Startを押してください。"
                    Exit Select
                Case enState.WaitRivalStart
                    '対戦相手のスタート待ち
                    Me.lblMessage.Text = "対戦相手のStartを待っています。"
                    Exit Select
                Case enState.StartReady
                    If _Rival Is Nothing OrElse _Own Is Nothing Then
                        Return
                    End If
                    If Not _Rival.Ready Then
                        Return
                    End If
                    If Not _Own.Ready Then
                        Return
                    End If
                    Me.pnlSetting.Visible = False
                    Me.StartCountDown()
                    Exit Select
                Case enState.PlayingGame
                    'ゲーム開始
                    Me.GameStart()
                    Exit Select
                Case enState.GameSet
                    Me.GameSet()
                    Exit Select
            End Select
            Me._State = value
        End Set
    End Property



    Private Sub frmStart_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
        Me.Text = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name
        Me.Text += " " & Convert.ToString(System.Reflection.Assembly.GetExecutingAssembly().GetName().Version)

        _Grid = New clsGrid(Me.picGrid)
        _Grid.ParentForm = Me
        Me.SetState(enState.Disconnected)
    End Sub



    Private Sub Init_pnlSetting()
        Me.pnlSetting.Visible = True
        Me.pnlSetting.BringToFront()
        '最前面に表示
        Me.btnStartCancel.Text = "START"
        'パネルの場所とサイズをGridに合わせる
        Me.pnlSetting.Left = Me.picGrid.Left + clsConst.CellSize.Width
        Me.pnlSetting.Top = Me.picGrid.Top + clsConst.CellSize.Height
        Me.pnlSetting.Size = New Size(clsConst.CellSize.Width * 6, RealTimeChessFrameWork.clsConst.CellSize.Height * 6)
        Me.btnStartCancel.Enabled = False
        Me.radServer.Enabled = True
        Me.radClient.Enabled = True
        radServer.Checked = False
        radClient.Checked = False
        txtIP.Enabled = False
        txtPort.Enabled = False
        txtIP.Text = My.Settings.LastIP
        txtPort.Text = My.Settings.LastPort.ToString()
    End Sub



    Private Sub rad_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles radServer.CheckedChanged, radClient.CheckedChanged
        Dim rad As RadioButton = DirectCast(sender, RadioButton)
        If Not rad.Checked Then
            Return
        End If
        If rad.Text = "Server" Then
            txtIP.Enabled = False
            txtIP.Text = "LocalHost"
            txtPort.Enabled = True
        Else
            txtIP.Enabled = True
            txtIP.Text = My.Settings.LastIP
            txtPort.Enabled = True
        End If
        Me.btnStartCancel.Enabled = True
        Me.lblMessage.Text = "IP,Portを入力し、" & vbCr & vbLf & "STARTを押してください。"

    End Sub



    Private Sub StartCountDown()
        AddHandler Me._Tim.Tick, New EventHandler(AddressOf Tim_Tick)
        Me._Tim.Interval = 1000
        Me._CountDown = 4
        AddHandler Me._Tim2.Tick, New EventHandler(AddressOf Tim2_Tick)
        Me._Tim2.Interval = 2
        Me._Tim.Start()
    End Sub



    Private Sub Tim_Tick(ByVal sender As Object, ByVal e As EventArgs)
        If _CountDown < 0 Then
            _Tim.[Stop]()
            _Tim2.[Stop]()
            RemoveHandler _Tim.Tick, New EventHandler(AddressOf Tim_Tick)
        Else
            _CountDown -= 1
            If _CountDown = 0 Then
                'this.State = enState.PlayingGame;
                Me.SetState(enState.PlayingGame)
            End If
            Me._FontDepth = 255
            _Tim2.Start()
        End If
    End Sub



    Private Sub Tim2_Tick(ByVal sender As Object, ByVal e As EventArgs)
        If Me._FontDepth >= 5 Then
            Me._FontDepth -= 5
        End If
        If _FontDepth = 0 Then
            _Tim2.[Stop]()
            RemoveHandler _Tim2.Tick, New EventHandler(AddressOf Tim_Tick)
        End If
        Me.picGrid.Refresh()
    End Sub



    Private Sub GameStart()
        If _tcp.Mode = TcpConnection.enMode.server Then
            'サーバーは黒
            _Own.NewPieces(PieceColor.Black, PieceDirection.Up, Me._Grid)
            _Own.PutPieces(Me._Grid)
            _Rival.NewPieces(PieceColor.White, PieceDirection.Down, Me._Grid)
            _Rival.PutPieces(Me._Grid)
        Else
            'クライアントは白
            _Own.NewPieces(PieceColor.White, PieceDirection.Up, Me._Grid)
            _Own.PutPieces(Me._Grid)
            _Rival.NewPieces(PieceColor.Black, PieceDirection.Down, Me._Grid)
            _Rival.PutPieces(Me._Grid)
        End If
    End Sub



    Private GameSetCount As Integer = 0
    Private Sub GameSet()
        Me.Refresh()

        GameSetCount = 0
        Dim Tim3 As New Timer()
        Tim3.Interval = 3
        AddHandler Tim3.Tick, New EventHandler(AddressOf Tim3_Tick)
        Tim3.Start()
    End Sub



    Private Sub Tim3_Tick(ByVal sender As Object, ByVal e As EventArgs)
        If Me._Own Is Nothing Then

            DirectCast(sender, Timer).[Stop]()
        End If
        Me.GameSetCount += 7
        If Me.GameSetCount > 200 Then
            DirectCast(sender, Timer).[Stop]()

            _Grid.AllCellPiecesClear()
            'Cell内のPieceの参照を削除
            _Own = Nothing
            'Playerを削除
            _Rival = Nothing
            '対戦相手を削除

            Me.SetState(enState.WaitOwnStart)
        End If
        Me.Refresh()
    End Sub



    Private Sub picGrid_Paint(ByVal sender As Object, ByVal e As PaintEventArgs) Handles picGrid.Paint
        If Me._Grid IsNot Nothing Then
            Me._Grid.Draw(e.Graphics)
        End If
        If Me._Own IsNot Nothing Then
            _Own.Draw(e.Graphics)
        End If
        If Me._Rival IsNot Nothing Then
            _Rival.Draw(e.Graphics)
        End If

        If Me.State = enState.StartReady Then
            If Me._CountDown > -1 Then
                Dim fontSize As Integer = 70
                Dim font As New Font("ＭＳ Ｐ ゴシック", fontSize)
                Dim pen As New Pen(Color.FromArgb(_FontDepth, Color.Black))
                Dim brush As New SolidBrush(Color.FromArgb(_FontDepth, Color.Black))
                Dim p As New PointF(picGrid.Width \ 2, picGrid.Height \ 2 - fontSize)
                Dim f As New StringFormat()
                f.Alignment = StringAlignment.Center
                If Me._CountDown = 0 Then
                    e.Graphics.DrawString("START", font, brush, p, f)
                Else
                    e.Graphics.DrawString(_CountDown.ToString(), font, brush, p, f)
                    'e.Graphics.FillEllipse(Brushes.Red, p.X - 2, p.Y - 2, 4, 4);
                End If
            End If
        End If
        If Me.State = enState.GameSet AndAlso Me._Own IsNot Nothing Then
            Dim text As String = String.Empty
            Dim brush As SolidBrush
            If Me._Own.IsWinner Then
                Dim c As Color = Color.FromArgb(Me.GameSetCount, Color.White)
                e.Graphics.FillRectangle(New SolidBrush(c), 0, 0, picGrid.Width, picGrid.Height)
                text = "YOU WIN !!"
                brush = New SolidBrush(Color.Black)
            Else
                Dim c As Color = Color.FromArgb(Me.GameSetCount, Color.Red)
                e.Graphics.FillRectangle(New SolidBrush(c), 0, 0, picGrid.Width, picGrid.Height)
                text = "YOU LOSE"
                brush = New SolidBrush(Color.Red)
            End If
            Dim fontSize As Integer = 50
            Dim font As New Font("ＭＳ Ｐ ゴシック", fontSize)
            Dim pen As New Pen(Color.FromArgb(_FontDepth, Color.Black))
            Dim p As New PointF(picGrid.Width \ 2, picGrid.Height \ 2 - fontSize * 2)
            Dim f As New StringFormat()
            f.Alignment = StringAlignment.Center

            e.Graphics.DrawString(text, font, brush, p, f)
        End If

    End Sub



    Private Sub button1_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.SetState(enState.StartReady)
    End Sub


#Region "Pieceの移動"

    Private havePiece As clsPiece
    Private Sub picGrid_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs) Handles picGrid.MouseDown

        If Me._Grid Is Nothing Then
            Return
        End If
        'ピースがあれば持つ
        Dim x As Integer = e.X \ clsConst.CellSize.Width
        Dim y As Integer = e.Y \ clsConst.CellSize.Height

        'System.Diagnostics.Debug.Print(e.Location.ToString() & " -> " & x.ToString() & " " & y.ToString())

        If Me._Grid(x, y).Piece IsNot Nothing Then
            Me.havePiece = _Grid(x, y).Piece
            '自分の駒でなければ何もしない
            If Not Me.havePiece.Owner.IsMovable Then
                Me.havePiece = Nothing
            End If
        End If
        If Me.havePiece Is Nothing Then
            Return
        End If

        '動かせなければ何もしない
        If Not Me.havePiece.IsMovable Then
            Return
        End If
        Me.havePiece.IsDragging = True
        Me.havePiece.BeforeMovePoint = New Point(e.X, e.Y)
        Me.havePiece.CalculateMovableArea()
        Me.havePiece.ShowMovableArea()
        Me.picGrid.Refresh()
    End Sub

    Private Sub picGrid_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs) Handles picGrid.MouseMove
        If Me.havePiece Is Nothing Then
            Return
        End If
        'ドラッグしていなければ何もしない
        If Not Me.havePiece.IsDragging Then
            Return
        End If
        Me.havePiece.Left = e.X - clsConst.CellSize.Width \ 2
        Me.havePiece.Top = e.Y - clsConst.CellSize.Height \ 2
    End Sub

    Private Sub picGrid_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs) Handles picGrid.MouseUp
        If Me.havePiece Is Nothing Then
            Return
        End If

        Me.havePiece.IsDragging = False
        Dim x As Integer = (Me.havePiece.Left + clsConst.CellSize.Width \ 2) \ clsConst.CellSize.Width
        Dim y As Integer = (Me.havePiece.Top + clsConst.CellSize.Height \ 2) \ clsConst.CellSize.Height
        Dim NowPoint As New Point(x, y)
        If x > 7 OrElse y > 7 OrElse x < 0 OrElse y < 0 OrElse (Not Me.havePiece.IsMovablePoint(NowPoint) AndAlso Not Me.havePiece.IsGettable(NowPoint)) Then
            'はみ出ている場合　or　置けない場合は元に戻す
            Me.havePiece.Left = Me.havePiece.Place.X * clsConst.CellSize.Width
            Me.havePiece.Top = Me.havePiece.Place.Y * clsConst.CellSize.Height
        Else
            'はみ出ていなければそこに移動する
            Me.havePiece.Left = NowPoint.X * clsConst.CellSize.Width
            Me.havePiece.Top = NowPoint.Y * clsConst.CellSize.Height

            _Grid.MovePiece(Me.havePiece, NowPoint)
        End If
        'Cellの色分けを消す
        Me.havePiece.Grid.AllCellClear()
        Me.havePiece = Nothing
    End Sub
#End Region

    Private Sub frmStart_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles MyBase.FormClosing
        'TcpConnectionを使っていれば削除
        If _tcp IsNot Nothing Then
            _tcp.Dispose()
            _tcp = Nothing
        End If
        '使用したIP,ポートを記憶
        My.Settings.LastIP = Me.txtIP.Text
        Try
            My.Settings.LastPort = Integer.Parse(Me.txtPort.Text)
        Catch
            My.Settings.LastPort = 9999
        End Try
        My.Settings.Save()
    End Sub


    Private Sub btnStart_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnStartCancel.Click
        Select Case Me.State
            Case enState.Disconnected
                If radServer.Checked Then
                    Me.btnStartCancel.Enabled = False
                    _tcp = New TcpConnection()
                    _tcp.Mode = TcpConnection.enMode.server
                    _tcp.IP = txtIP.Text
                    _tcp.Port = Integer.Parse(txtPort.Text)
                    AddHandler _tcp.ServerStarted, AddressOf tcp_ServerStarted
                    AddHandler _tcp.ServerStartFailure, AddressOf tcp_ServerStartFailure
                    AddHandler _tcp.ClientConnected, AddressOf tcp_ClientConnected
                    AddHandler _tcp.MessageReceived, AddressOf tcp_MessageReceived
                    _tcp.Start()
                Else
                    Me.btnStartCancel.Enabled = False
                    _tcp = New TcpConnection()
                    _tcp.Mode = TcpConnection.enMode.client
                    _tcp.IP = txtIP.Text
                    _tcp.Port = Integer.Parse(txtPort.Text)
                    AddHandler _tcp.ServerConnectSuccess, AddressOf tcp_ServerConnectSuccess
                    AddHandler _tcp.ServerConnectFailure, AddressOf tcp_ServerConnectFailure
                    AddHandler _tcp.MessageReceived, AddressOf tcp_MessageReceived
                    _tcp.Start()
                End If
                Exit Select
            Case enState.WaitClient
                Me.SetState(enState.Disconnected)
                Exit Select
            Case enState.WaitOwnStart
                Me.btnStartCancel.Enabled = False
                _Own = New clsPlayer("わたし", True)
                _Own.Ready = True
                Me.SendMessage(enMsgKey.READY, String.Empty)
                If _Rival IsNot Nothing Then
                    Me.SetState(enState.StartReady)
                Else
                    Me.SetState(enState.WaitRivalStart)
                End If
                Exit Select
        End Select
    End Sub

    Private Sub tcp_MessageReceived(ByVal sender As Object, ByVal e As EventArgs)
        'データ受信
        Dim m As clsMessage = _tcp.NextMessage
        If m IsNot Nothing Then
            Dim key As enMsgKey = CType([Enum].Parse(GetType(enMsgKey), m.Key), enMsgKey)
            Me.MessageProcessing(key, m.Value)
        End If
    End Sub

    Private Sub tcp_ServerStartFailure(ByVal sender As Object, ByVal e As EventArgs)
        'サーバー立ち上げ失敗
        Me.SetState(enState.Disconnected)
        Me.lblMessage.Text = "Serverの立ち上げに失敗しました。"
    End Sub

    Private Sub tcp_ServerStarted(ByVal sender As Object, ByVal e As EventArgs)
        'サーバー立ち上げ成功
        Me.SetState(enState.WaitClient)
    End Sub

    Private Sub tcp_ClientConnected(ByVal sender As Object, ByVal e As EventArgs)
        'クライアントが接続した。
        Me.SetState(enState.WaitOwnStart)
    End Sub

    Private Delegate Sub SetStateDelegate(ByVal state As enState)

    Private Sub SetState(ByVal state As enState)
        If InvokeRequired Then
            Invoke(New SetStateDelegate(AddressOf SetState), New Object() {state})
        Else
            Me.State = state
        End If
    End Sub



    Private Sub tcp_ServerConnectSuccess(ByVal sender As Object, ByVal e As EventArgs)
        'サーバー接続成功
        Me.SetState(enState.WaitOwnStart)
    End Sub

    Private Sub tcp_ServerConnectFailure(ByVal sender As Object, ByVal e As EventArgs)
        'サーバー接続失敗
        Me.SetState(enState.Disconnected)
        Me.lblMessage.Text = "Serverへの接続に失敗しました。"
    End Sub



    Public Sub SendMessage(ByVal key As enMsgKey, ByVal value As String)
        If _tcp Is Nothing Then
            Throw New Exception("TcpConnectionが開いていないのでメッセージは送れません。")
        End If
        Dim m As New clsMessage()
        m.Key = key.ToString()
        m.Value = value
        _tcp.Send(m)
    End Sub

    Public Delegate Sub MessageProcessingDelegate(ByVal key As enMsgKey, ByVal value As String)

    Private Sub MessageProcessing(ByVal key As enMsgKey, ByVal value As String)
        If InvokeRequired Then
            Invoke(New MessageProcessingDelegate(AddressOf MessageProcessing), New Object() {key, value})
        Else

            Select Case Me.State
                Case enState.Disconnected
                    '無視
                    Exit Select
                Case enState.WaitClient
                    '無視
                    Exit Select
                Case enState.WaitOwnStart
                    If key = enMsgKey.READY Then
                        _Rival = New clsPlayer("らいばる", False)
                        _Rival.Ready = True
                        Me.SetState(enState.StartReady)
                    End If
                    Exit Select
                Case enState.WaitRivalStart
                    If key = enMsgKey.READY Then
                        _Rival = New clsPlayer("らいばる", False)
                        _Rival.Ready = True

                        Me.SetState(enState.StartReady)
                    End If
                    Exit Select
                Case enState.PlayingGame
                    If key = enMsgKey.MOVE Then
                        '敵の駒が動いた
                        Dim ar As String() = value.Split(","c)
                        Dim idx As Integer = Integer.Parse(ar(0))
                        Dim x As Integer = Integer.Parse(ar(1))
                        Dim y As Integer = Integer.Parse(ar(2))
                        Dim p As clsPiece = _Rival.Pieces(idx)
                        x = clsConst.XCount - 1 - x
                        y = clsConst.YCount - 1 - y
                        _Grid.MovePiece(p, New Point(x, y))
                    ElseIf key = enMsgKey.[GET] Then
                        '自分の駒がとられた
                        Dim p As clsPiece = _Own.Pieces(Integer.Parse(value))
                        p.Parent.Controls.Remove(p)
                    ElseIf key = enMsgKey.WIN Then
                        '自分のキングが取られた
                        Me._Own.IsWinner = False
                        Me.SetState(enState.GameSet)
                    End If
                    Exit Select
            End Select
        End If
    End Sub

End Class
