﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text


Public Class TcpConnection
    Implements IDisposable

    ''' <summary>
    ''' 起動モード
    ''' </summary>
    Public Enum enMode
        server = 0
        client
    End Enum

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    Public Sub New()
        form = New clsTcpConnection()
    End Sub


    'デバッグ時に表示するフォーム。通信もこいつがやる。
    Private form As clsTcpConnection

    '受信したデータを保持するキュー。
    Private _ReceiveData As New Queue(Of clsMessage)()

    ''' <summary>
    ''' 実行モードを選択してください。
    ''' サーバー/クライアント
    ''' </summary>
    Public Property Mode() As enMode
        Get
            Return form.Mode
        End Get
        Set(ByVal value As enMode)
            form.Mode = value
        End Set
    End Property

    ''' <summary>
    ''' 接続先IPです。
    ''' サーバーの場合は無視されます。
    ''' クライアントの場合は接続先IPを指定してください。
    ''' </summary>
    Public Property IP() As String
        Get
            Return form.IP
        End Get
        Set(ByVal value As String)
            form.IP = value
        End Set
    End Property

    ''' <summary>
    ''' 待ち受け/接続先ポートです。
    ''' 既定値は9999です。
    ''' </summary>
    Public Property Port() As Integer
        Get
            Return form.Port
        End Get
        Set(ByVal value As Integer)
            form.Port = value
        End Set
    End Property

    ''' <summary>
    ''' 名前です。
    ''' チャットとかゲームとか。
    ''' </summary>
    Public Property Name() As String
        Get
            Return form.Name
        End Get
        Set(ByVal value As String)
            '使えない名前だった場合例外スロー
            If Not clsCommon.nameCheck(value) Then
                Throw New NotUsableNameException(value)
            Else
                form.Name = value
            End If
        End Set
    End Property

    ''' <summary>
    ''' 受信メッセージのQueue
    ''' </summary>
    Public ReadOnly Property ReceivedMessages() As Queue(Of clsMessage)
        Get
            Return _ReceiveData
        End Get
    End Property

    ''' <summary>
    ''' 受信しているメッセージのうち、一番最初のものを取り出す。
    ''' メッセージがなければNullが返る
    ''' </summary>
    Public ReadOnly Property NextMessage() As clsMessage
        Get
            If Me.ReceivedMessages.Count = 0 Then
                Return Nothing
            End If
            Return Me.ReceivedMessages.Dequeue()
        End Get
    End Property

    Public Function Menbers() As List(Of String)
        Return form.Menbers()
    End Function

    ''' <summary>
    ''' 通信を開始します。
    ''' 終了時にはかならず、Disposeを呼び出してください。
    ''' </summary>
    Public Sub Start()
        form.Parent = Me
        form.Start()
    End Sub

    ''' <summary>
    ''' 通信を終了して破棄します。
    ''' </summary>
    Public Sub Dispose() Implements IDisposable.Dispose
        form.Dispose()
    End Sub

    ''' <summary>
    ''' メッセージを送信します。
    ''' </summary>
    Public Sub Send(ByVal m As clsMessage)
        form.Send(m)
    End Sub


    ''' <summary>
    ''' formより呼び出し。メッセージを渡すため。
    ''' </summary>
    Friend Sub AddMessage(ByVal m As clsMessage)
        If m.SendTo = Me.Name OrElse m.SendTo = String.Empty Then
            Me._ReceiveData.Enqueue(m)
            Me.OnReceiveMessage(New EventArgs())
        End If
    End Sub

    '共通イベント
    ''' <summary>
    ''' メッセージを受信しました。
    ''' </summary>
    Public Event MessageReceived As EventHandler

    Friend Overridable Sub OnReceiveMessage(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  OnMessageReceived")

        'System.Diagnostics.Debug.Print("--- Raise Event ---  MessageReceived")
        RaiseEvent MessageReceived(Me, e)
    End Sub

    ''' <summary>
    ''' メンバー構成が変わりました。
    ''' </summary>
    Public Event MenberChanged As EventHandler
    Friend Overridable Sub OnMenberChanged(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  MenberChanged")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  MenberChanged")
        RaiseEvent MenberChanged(Me, e)
    End Sub


    'メッセージ送信しました。

    'サーバーイベント
    ''' <summary>
    ''' サーバー立ち上げ成功
    ''' </summary>
    Public Event ServerStarted As EventHandler
    Friend Overridable Sub OnServerStarted(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  ServerStarted")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  ServerStarted")
        RaiseEvent ServerStarted(Me, e)
    End Sub
    ''' <summary>
    ''' サーバー立ち上げ失敗
    ''' </summary>
    Public Event ServerStartFailure As EventHandler
    Friend Overridable Sub OnServerStartFailure(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  ServerStartFailure")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  ServerStartFailure")
        RaiseEvent ServerStartFailure(Me, e)
    End Sub

    ''' <summary>
    ''' クライアントが接続しました。
    ''' </summary>
    Public Event ClientConnected As EventHandler
    Friend Overridable Sub OnClientConnected(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  ClientConnected")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  ClientConnected")
        RaiseEvent ClientConnected(Me, e)
    End Sub
    ''' <summary>
    ''' クライアントが切断しました。
    ''' </summary>
    Public Event ClientDisconnected As EventHandler
    Friend Overridable Sub OnClientDisconnected(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  ClientDisconnected")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  ClientDisconnected")
        RaiseEvent ClientDisconnected(Me, e)
    End Sub

    'クライアントイベント
    ''' <summary>
    ''' サーバーに接続しました。
    ''' </summary>
    Public Event ServerConnectSuccess As EventHandler
    Friend Overridable Sub OnServerConnectSuccess(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  ServerConnectSuccess")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  ServerConnectSuccess")
        RaiseEvent ServerConnectSuccess(Me, e)
    End Sub

    ''' <summary>
    ''' サーバーに接続失敗しました。
    ''' </summary>
    Public Event ServerConnectFailure As EventHandler
    Friend Overridable Sub OnServerConnectFailure(ByVal e As EventArgs)
        System.Diagnostics.Debug.Print("--- Call ---  ServerConnectFailure")
        '    System.Diagnostics.Debug.Print("--- Raise Event ---  ServerConnectFailure")
        RaiseEvent ServerConnectFailure(Me, e)
    End Sub

End Class
