﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Net.Sockets
Imports System.Net
Imports System.Threading


Friend Class clsClientHandler

    Private _OwnerName As String = String.Empty         'ClientHandlerの作成者の名前
    Private _ConnectedTo As String = String.Empty       '接続先の名前
    Private buffer As Byte()                            '受信データ
    Private socket As Socket                            '保持Socket
    Private nwStream As NetworkStream                   'NetworkStream
    Private cbRead As AsyncCallback                     'Read用Callback
    Private cbWrite As AsyncCallback                    'Write用Callback
    Private clsServer As clsTcpConnection = Nothing     '親フォームの参照

    ''' <summary>
    ''' Socketクラスのハンドルを返す
    ''' </summary>
    Public ReadOnly Property ClientHandle() As IntPtr
        Get
            Return socket.Handle
        End Get
    End Property

    Public Property OwnerName() As String
        Get
            Return _OwnerName
        End Get
        Set(ByVal value As String)
            _OwnerName = value
        End Set
    End Property
    Public Property ConnectedTo() As String
        Get
            Return _ConnectedTo
        End Get
        Set(ByVal value As String)
            _ConnectedTo = value
        End Set
    End Property

    ''' <summary>
    ''' ClientHandlerにSocketを保持します。
    ''' </summary>
    ''' <param name="socketForClient">通信を確立したSocket</param>
    ''' <param name="_frmServer">親フォーム</param>
    ''' <param name="owner">Socketを保持している人の名前(ゲーム用)</param>
    Public Sub New(ByVal socketForClient As Socket, ByVal _frmServer As clsTcpConnection, ByVal owner As String)
        socket = socketForClient
        clsServer = _frmServer
        buffer = New Byte(255) {}
        nwStream = New NetworkStream(socketForClient)
        cbRead = New AsyncCallback(AddressOf Me.OnReadComplete)
        cbWrite = New AsyncCallback(AddressOf Me.OnSendComplete)
        OwnerName = owner
    End Sub

    ''' <summary>
    ''' ClientHandlerにSocketを保持します。
    ''' </summary>
    ''' <param name="socketForClient">通信を確立したSocket</param>
    ''' <param name="_frmServer">親フォーム</param>
    Public Sub New(ByVal socketForClient As Socket, ByVal _frmServer As clsTcpConnection)
        Me.New(socketForClient, _frmServer, String.Empty)
    End Sub


    ''' <summary>
    ''' 受信開始
    ''' </summary>
    Public Sub StartRead()
        nwStream.BeginRead(buffer, 0, buffer.Length, cbRead, Nothing)
    End Sub

    ''' <summary>
    ''' 受信完了
    ''' dlgWriteTextにて親フォームへ通知される
    ''' </summary>
    Private Sub OnReadComplete(ByVal ar As IAsyncResult)
        Try
            If nwStream Is Nothing Then
                Return
            End If

            Dim bytesRead As Integer = nwStream.EndRead(ar)

            If bytesRead > 0 Then
                Dim getByte As Byte() = New Byte(bytesRead - 1) {}
                For i As Integer = 0 To bytesRead - 1
                    getByte(i) = buffer(i)
                Next

                Dim m As clsMessage = clsCommon.GetMessageFromBytes(getByte)

                clsServer.ReceiveMessage(Me, m)
                Me.StartRead()
            Else
                '終了ボタンが押された場合はここに来る
                clsServer.DeleteClient(Me)
                nwStream.Close()
                socket.Close()
                nwStream = Nothing
                Thread.Sleep(20)
                socket = Nothing
            End If
        Catch generatedExceptionName As System.Net.Sockets.SocketException
            'スレッドが閉じられた時に発生
            Return
        Catch generatedExceptionName As System.IO.IOException
            'スレッドが閉じられた時に発生
            Return
        Catch ex As Exception
            'エラーログの書き込み
            clsServer.WriteLog("受信エラーが起こりました " & ex.ToString())
        End Try
    End Sub

    ''' <summary>
    ''' 送信開始
    ''' </summary>
    ''' <param name="buffer">送信Bytes</param>
    Public Sub SendString(ByVal buffer As Byte())
        nwStream.BeginWrite(buffer, 0, buffer.Length, cbWrite, Nothing)
    End Sub

    ''' <summary>
    ''' クライアントに切断情報を送るため
    ''' </summary>
    Public Sub Disconnect()
        socket.Disconnect(False)
    End Sub


    ''' <summary>
    ''' 送信完了
    ''' 完了後、再び受信開始
    ''' </summary>
    Private Sub OnSendComplete(ByVal ar As IAsyncResult)
        nwStream.EndWrite(ar)
        Me.StartRead()
    End Sub

End Class
