﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading



#Region "Delegate"
''' <summary>
''' メッセージ受信用デリゲート
''' </summary>
''' <param name="cl"></param>
''' <param name="m"></param>
Friend Delegate Sub dlgReceiveMessage(ByVal cl As clsClientHandler, ByVal m As clsMessage)
''' <summary>
''' dlgSetList
''' ClientHandlerを渡す
''' </summary>
Friend Delegate Sub dlgSetList(ByVal ch As clsClientHandler)
''' <summary>
''' dlgWriteText
''' ClientHandlerと文字列
''' </summary>
''' <param name="ch"></param>
''' <param name="text"></param>
Friend Delegate Sub dlgWriteText(ByVal ch As clsClientHandler, ByVal text As String)
''' <summary>
''' dlgWriteLog
''' 文字列
''' </summary>
''' <param name="text"></param>
Friend Delegate Sub dlgWriteLog(ByVal text As String)
#End Region

Friend Class clsTcpConnection
    Implements IDisposable



#Region "Constractor"
    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    Public Sub New()

        'デフォルト設定
        'TcpConnectionのプロパティを設定しなかった場合の値
        Me.Mode = TcpConnection.enMode.server        'サーバーとして起動
        Me.IP = "localhost"        '自PCに接続
        Me.Port = 9999        'ポートは9999
        Me.Name = "nanashi"        '名前 nanashi

    End Sub

#End Region

#Region "Private Variables"

    Private _parent As TcpConnection
    Private _mode As TcpConnection.enMode    'サーバーorクライアント
    Private _ip As String    '接続先IP　クライアントのときに使用
    Private _port As Integer    '待ち受け(接続)先ポート
    Private _name As String    '名前

    'サーバーとして起動時に使用する変数
    Private listener As TcpListener = Nothing    'クライアント待ち受けListener
    Private threadServer As Thread = Nothing    '別スレッドでクライアント待ち受けを続けるThread
    Private listClientHandler As List(Of clsClientHandler) = Nothing    'クライアントを保持するリスト

    'クライアントとして起動時に使用する変数
    Private client As TcpClient = Nothing    'サーバーに接続するクライアント
    Private threadClient As Thread = Nothing    'データの受信を待ち受けるThread
    Private _menbers As New List(Of String)()    '接続しているメンバ


#End Region

#Region "Properties"

    Public Shadows Property Parent() As TcpConnection
        Get
            Return _parent
        End Get
        Set(ByVal value As TcpConnection)
            _parent = value
        End Set
    End Property
    ''' <summary>
    ''' 起動モード
    ''' サーバーorクライアント
    ''' </summary>
    Public Property Mode() As TcpConnection.enMode
        Get
            Return _mode
        End Get
        Set(ByVal value As TcpConnection.enMode)
            _mode = value
        End Set
    End Property

    ''' <summary>
    ''' 接続先IP
    ''' クライアントモードのときのみ使用される
    ''' </summary>
    Public Property IP() As String
        Get
            Return _ip
        End Get
        Set(ByVal value As String)
            _ip = value
        End Set
    End Property

    ''' <summary>
    ''' 待ち受けポート/接続ポート
    ''' </summary>
    Public Property Port() As Integer
        Get
            Return _port
        End Get
        Set(ByVal value As Integer)
            _port = value
        End Set
    End Property

    ''' <summary>
    ''' 名前
    ''' ゲームとかチャットの識別にどうぞ
    ''' </summary>
    Public Shadows Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            _name = value
        End Set
    End Property

    '''' <summary>
    '''' デバッグモード
    '''' trueにするとフォームが表示される。
    '''' メッセージの手動送信とかが可能に。
    '''' </summary>
    'Public Property DebugMode() As Boolean
    '    Get
    '        Return _debugMode
    '    End Get
    '    Set(ByVal value As Boolean)
    '        _debugMode = value
    '    End Set
    'End Property

#End Region

#Region "Public Functions"

    ''' <summary>
    ''' Start
    ''' 開始前の表示設定
    ''' </summary>
    Public Sub Start()
        '開始！！
        StartSock()
    End Sub

    ''' <summary>
    ''' Stop()
    ''' サーバー/クライアントの停止
    ''' </summary>
    Public Sub [Stop]()
        If listener IsNot Nothing Then
            CloseServer()
        End If
        If client IsNot Nothing Then
            CloseClient()
        End If
    End Sub

#End Region

    Public Sub Send(ByVal m As clsMessage)
        Dim data As Byte() = clsCommon.GetBytesFromMessage(m)
        If Me.Mode = TcpConnection.enMode.server Then
            'サーバーの場合の送り方
            If m.SendTo <> String.Empty Then
                '送信先が指定されている＝指定先に送信
                For Each cl As clsClientHandler In listClientHandler
                    If cl.ConnectedTo = m.SendTo Then
                        Me.sendData(cl, data)
                    End If
                Next
            Else
                '送信先が指定されていない＝全員に送信
                'サーバーが転送してきている場合もあるので、Fromの人には送らない
                For Each cl As clsClientHandler In listClientHandler
                    If cl.ConnectedTo <> m.ReceiveFrom Then
                        Me.sendData(cl, data)
                    End If
                Next
            End If
        Else
            'クライアントの場合はToに関係なくとりあえず送る
            Me.sendData(listClientHandler(0), data)
        End If
    End Sub

    Public Function Menbers() As List(Of String)
        Dim items As New List(Of String)()
        For Each item As String In _menbers
            items.Add(item.ToString())
        Next
        Return items
    End Function




#Region "Private Functions (Common)"

    ''' <summary>
    ''' Modeを判断してサーバーorクライアントの開始
    ''' </summary>
    Private Sub StartSock()
        Select Case Mode
            Case TcpConnection.enMode.server
                If ServerStart() Then
                    'サーバーの開始
                    'サーバーが開始できたら
                    Me.Parent.OnServerStarted(New EventArgs())
                Else
                    'サーバー立ち上げ失敗
                    Me.Parent.OnServerStartFailure(New EventArgs())
                End If
                Exit Select
            Case TcpConnection.enMode.client
                If ClientStart() Then
                    'クライアントの開始
                    'クライアントが開始できたら
                    Me.Parent.OnServerConnectSuccess(New EventArgs())
                Else
                    'クライアント開始失敗
                    Me.Parent.OnServerConnectFailure(New EventArgs())
                End If
                Exit Select
        End Select
    End Sub

    ''' <summary>
    ''' データ送信
    ''' 指定のClientへdataを送る
    ''' </summary>
    Private Sub sendData(ByVal ch As clsClientHandler, ByVal data As Byte())
        Try
            ch.SendString(data)
        Catch
            MsgBox("送信できませんでした。", Nothing, "送信エラー")
            WriteLog("送信エラー:送信できませんでした。")
        End Try
    End Sub


#End Region


#Region "Private Functions (for Server)"
    ''' <summary>
    ''' サーバーの開始
    ''' </summary>
    ''' <returns>成否</returns>
    Private Function ServerStart() As Boolean
        Try
            WriteLog("サーバー開始")
            listClientHandler = New List(Of clsClientHandler)()

            '別スレッドにてクライアント待ち受け開始
            threadServer = New Thread(New ThreadStart(AddressOf ServerListen))
            threadServer.Start()
            Return True
        Catch ex As Exception
            WriteLog("サーバー接続エラー:" & ex.Message)
            listener.[Stop]()
            Return False
        End Try
    End Function

    ''' <summary>
    ''' ServerListen()
    ''' クライアントの待ち受け用無限ループ
    ''' threadServerにて動作
    ''' </summary>
    Private Sub ServerListen()
        'TcpListenerを生成
        listener = New TcpListener(IPAddress.Any, Me.Port)
        '自PC、指定Port
        Try
            'クライアントの接続を受けるための永久ループ
            listener.Start()
            While True
                '接続が来るまで待機
                Dim socketForClient As Socket = listener.AcceptSocket()
                '接続されたSocketを使ってClientHandlerクラスを生成
                Dim handler As New clsClientHandler(socketForClient, Me, Me.Name)
                '本スレッドに通知
                Me.AddNewClient(handler)
                'this.BeginInvoke(new dlgSetList(this.AddNewClient), new object[] { handler });
                'ClientHandler非同期受信開始
                handler.StartRead()
            End While
        Catch generatedExceptionName As System.Threading.ThreadAbortException
            'スレッドが閉じられた時に発生
            Return
        Catch generatedExceptionName As System.Net.Sockets.SocketException
            'スレッドが閉じられた時に発生
            Return
        Catch ex As Exception
            'デリゲートでエラーを書き込む
            WriteLog("受信エラー" & ex.ToString())
            Return
        End Try
    End Sub

    ''' <summary>
    ''' サーバーの停止
    ''' </summary>
    Private Sub CloseServer()
        For Each cl As clsClientHandler In listClientHandler
            'これがないとクライアント側のDeleteClientが呼ばれない
            cl.Disconnect()
        Next
        'サーバーのインスタンスが有って、接続されていたら
        If listener IsNot Nothing Then
            listener.[Stop]()
            Thread.Sleep(20)
            listener = Nothing
            'ログを書き込む
            WriteLog("サーバーが閉じられました。")
        End If

        'スレッドは必ず終了させること
        If threadServer IsNot Nothing Then
            threadServer.Abort()
            threadServer = Nothing
        End If
    End Sub
#End Region


#Region "Private Functions (for Client)"
    ''' <summary>
    ''' ClientStart()
    ''' クライアントの開始
    ''' </summary>
    ''' <returns>成否</returns>
    Private Function ClientStart() As Boolean
        listClientHandler = New List(Of clsClientHandler)()
        Try
            'クライアントのソケットを用意
            client = New TcpClient(Me.IP, Me.Port)

            'つながったSocketを使ってClientHandlerクラスを生成
            Dim handler As New clsClientHandler(client.Client, Me, Me.Name)

            Me.AddNewClient(handler)

            'ClientHandler非同期受信開始
            handler.StartRead()

            WriteLog("クライアント接続接続されました:")
            Return (True)
        Catch ex As Exception
            WriteLog("クライアント接続エラー:" & ex.Message.ToString())
            Return (False)
        End Try

    End Function

    ''' <summary>
    ''' クライアントを閉じる
    ''' </summary>
    Private Sub CloseClient()
        'クライアントのインスタンスが有って、接続されていたら
        If client IsNot Nothing AndAlso client.Connected Then
            'ここでServerのDeleteClientが呼ばれるみたい。。。
            client.Close()
        End If
        'スレッドは必ず終了させること
        If threadClient IsNot Nothing Then
            threadClient.Abort()
        End If
        'ログを書き込む
        WriteLog("クライアントが閉じられました。")
    End Sub

#End Region


#Region "Private Functions (for Client)"

    ''' <summary>
    ''' メッセージ受信
    ''' </summary>
    Friend Sub ReceiveMessage(ByVal cl As clsClientHandler, ByVal m As clsMessage)
        '名前をClientHandlerに記入
        If m.Key = clsConst.DEFINE_KEY(clsConst.Define_SendName) Then
            cl.ConnectedTo = m.Value
            '名前をListboxに表示
            'resetClientListBox()
        End If
        '接続者リストを更新
        If m.Key = clsConst.DEFINE_KEY(clsConst.Define_SendMenbers) Then
            'Me.resetMenbersListBox(clsCommon.SepaleteString(m.Value, clsConst.SEPALETER))
            'イベント
            Me.Parent.OnMenberChanged(New EventArgs())
        End If

        '自分あての通常メッセージはTcpConnectionに渡す
        If Not m.SpecialKey Then
            If m.SendTo = Me.Name OrElse m.SendTo = String.Empty Then
                'ReceiveMessageイベントが発生する
                Me.Parent.AddMessage(m)
            End If
        End If

        'サーバーの場合は転送する
        If Me.Mode = TcpConnection.enMode.server Then
            'SpecialKeyのメッセージは送らない
            If Not m.SpecialKey Then
                'おくってきた人には送り返さない
                If m.SendTo <> m.ReceiveFrom Then
                    Me.Send(m)
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' 接続先追加
    ''' </summary>
    Friend Sub AddNewClient(ByVal cl As clsClientHandler)
        listClientHandler.Add(cl)
        'resetClientListBox()
        WriteLog(cl.ClientHandle.ToString() & " さんが接続しました。")

        sendSpecialItem_Name(cl)
        '接続先に自分の名前を送っておく
        Me.Parent.OnClientConnected(New EventArgs())
        'イベント

    End Sub

    ''' <summary>
    ''' 接続時に自動で送られるアイテム
    ''' 名前
    ''' </summary>
    Private Sub sendSpecialItem_Name(ByVal cl As clsClientHandler)
        Dim m As New clsMessage()
        m.SpecialKey = True
        m.Key = clsConst.DEFINE_KEY(clsConst.Define_SendName)
        m.Value = Me.Name
        Dim data As Byte() = clsCommon.GetBytesFromMessage(m)
        sendData(cl, data)

    End Sub

    ''' <summary>
    ''' 全員に今のメンバー状態を送る
    ''' Server専用
    ''' </summary>
    Private Sub sendSpecialItem_AllMenbers()
        If Me.Mode = TcpConnection.enMode.client Then
            Throw New Exception("サーバー専用メソッドです。")
        End If

        'メンバリスト作成
        Dim sb As New StringBuilder()
        'server名
        sb.Append(Me.Name + clsConst.SEPALETER)

        For Each s As String In _menbers
            sb.Append(s.ToString().Substring(clsConst.HandleID_Length).Trim() + clsConst.SEPALETER)
        Next
        sb.Length -= 1
        '最後の１文字削除
        Dim m As New clsMessage()
        m.SpecialKey = True
        m.Key = clsConst.DEFINE_KEY(clsConst.Define_SendMenbers)
        m.Value = sb.ToString()
        Me.Send(m)

        'サーバー自信のListboxも反映しておく
        'Me.resetMenbersListBox(clsCommon.SepaleteString(m.Value, clsConst.SEPALETER))

    End Sub

    ''' <summary>
    ''' 接続先削除
    ''' </summary>
    Friend Sub DeleteClient(ByVal cl As clsClientHandler)
        'Listクラスを総なめ
        For i As Integer = 0 To listClientHandler.Count - 1
            If listClientHandler(i) Is cl Then
                Me.Parent.OnClientDisconnected(New EventArgs())
                'イベント
                WriteLog(listClientHandler(i).ClientHandle.ToString() & " さんが切断しました。")
                listClientHandler.RemoveAt(i)
                'resetClientListBox()
                Return
            End If
        Next
    End Sub

    ''' <summary>
    ''' ログ書き出し
    ''' </summary>
    Friend Sub WriteLog(ByVal text As String)
        System.Diagnostics.Debug.Print(DateTime.Now.ToString() & "  " & text & vbCr & vbLf)
    End Sub

#End Region


#Region "Private Functions (for Debug)"

    '''' <summary>
    '''' クライアントを総なめし、ListBoxの内容を更新する
    '''' </summary>
    'Private Sub resetClientListBox()

    '    _menbers.Clear()

    '    For i As Integer = 0 To _menbers.Count - 2
    '        Dim id As String = clsCommon.GetSameLengthString(listClientHandler(i).ClientHandle.ToString(), clsConst.HandleID_Length)
    '        Dim name As String = listClientHandler(i).ConnectedTo
    '        _menbers.Add(id & name)
    '    Next

    '    '自分がサーバーなら全員にメンバーリストを送る
    '    If Me.Mode = TcpConnection.enMode.server Then
    '        Me.sendSpecialItem_AllMenbers()
    '    End If

    'End Sub

    'Private Sub resetMenbersListBox(ByVal menbers As List(Of String))
    '    System.Diagnostics.Debug.Print("### resetMenbersListBox ###")
    '    For Each menber As String In menbers
    '        System.Diagnostics.Debug.Print(menber)
    '    Next

    'End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        If listener IsNot Nothing Then
            CloseServer()
        End If
        If client IsNot Nothing Then
            CloseClient()
        End If
    End Sub

#End Region
End Class
